"""
"""
__version__ = '1.0'

import Globals
import OFS.SimpleItem
import OFS.PropertyManager
import OFS.Folder
import OFS.Image
import AccessControl.Role
import App.Undo
import urllib
import Persistence
import Acquisition
from DocumentTemplate import DT_HTML

manage_addContentIndexForm = Globals.HTMLFile('www/addContentIndex', globals())
def manage_addContentIndex(self, id, title='', folder='', submit='', REQUEST=None):
    """ Add a ContentIndex object """
    obj = ContentIndex(id, title=str(title), folder=str(folder))
    self._setObject(obj.getId(), obj)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u, urllib.quote(obj.getId()))
        REQUEST.RESPONSE.redirect(u+'/manage_workspace')
    return ''

class ContentIndex(
        OFS.PropertyManager.PropertyManager,
        Acquisition.Implicit,
        Persistence.Persistent,
        OFS.SimpleItem.SimpleItem,
        AccessControl.Role.RoleManager,
    ):
    """ A product to produce simple folder indexes """
    meta_type = "Content Index"
    id          = "ContentIndex"
    meta_type   = "Content Index"
    icon        = "misc_/ContentIndex/ContentIndex.gif"
    manage_main = OFS.PropertyManager.PropertyManager.manage_propertiesForm
    manage_options    = (
        OFS.PropertyManager.PropertyManager.manage_options
        +(
            {'label': 'View', 'action': 'preview',
	     'help': ('ContentIndex', 'preview.stx'), },
        )
        +App.Undo.UndoSupport.manage_options
        +AccessControl.Role.RoleManager.manage_options
    )
    __ac_permissions__ = (
        (
            ('View',
                ('index_html', '__call__', 'getIcon', 'preview', ),
                ('Anonymous', 'Manager', 'Owner', ),
            ),
        )
    )
    _properties=(
        {'id': 'title',     'type': 'string',   'mode': 'w'},
        {'id': 'folder',    'type': 'string',   'mode': 'w'},
        {'id': 'index',     'type': 'selection','mode': 'w', 'select_variable': 'indexes'},
        {'id': 'headers',   'type': 'boolean',  'mode': 'w'},
        {'id': 'reverse',   'type': 'boolean',  'mode': 'w'},
        {'id': 'hide_id',   'type': 'boolean',  'mode': 'w'},
        {'id': 'hide_title','type': 'boolean',  'mode': 'w'},
        {'id': 'hide_size', 'type': 'boolean',  'mode': 'w'},
        {'id': 'hide_date', 'type': 'boolean',  'mode': 'w'},
    )

    indexes = (
        'id',
        'title',
        'getSize',
        'bobobase_modification_time',
    )

    def __init__(self, id, title='', folder=''):
        """ Initialize a new ContentIndex object """
        self.id = id
        self.title = title
        self.folder = folder
        self.index = self.indexes[-1]
        self.headers = 1
        self.reverse = 1
        self.hide_id = 0
        self.hide_title = 0
        self.hide_size = 0
        self.hide_date = 0

    def _sorter(self, item1, item2):
        """ Compare two items based on the selected property """
        if hasattr(item1, self.index) and hasattr(item2, self.index):
            v1 = getattr(item1, self.index)
            if callable(v1):
                v1 = v1()
            v2 = getattr(item2, self.index)
            if callable(v2):
                v2 = v2()
            return cmp(v1, v2)
        return 0

    def _getList(self):
        """ """
        if not self.folder:
            folder = self.aq_parent
        else:
            folder = self.restrictedTraverse(self.folder)
        list = []
        for item in folder.objectValues():
            if  (
                    item.meta_type not in [ContentIndex.meta_type,
                                           ContentIndexIcons.meta_type] and
                    item.getId() not in ['standard_html_header',
                                         'standard_html_footer',
                                         'standard_html_error',
                                         'Control_Panel',
                                         'acl_users']
                ):
                list.append(item)
        list.sort(self._sorter)
        if self.reverse:
            list.reverse()
        return list

    def _internals(self):
        """ Return an un-rendered folder listing """ 
        dtml = '<table border="0" cellpadding="2" cellspacing="0" width="100%">\n' 
	dtml = dtml + ' <tr class="list-header">\n'
	dtml = dtml + '  <th align="left" valign="top" class="list-item" width="1%">Type</th>\n'
        if not self.hide_id: 
           dtml = dtml + '  <th align="left" valign="top" class="list-item">Id</th>\n'
        if not self.hide_title: 
            dtml = dtml + '  <th align="left" valign="top" class="list-item">Title</th>\n'
        if not self.hide_size: 
            dtml = dtml + '  <th align="right" valign="top" class="list-item">Size</th>\n'
        if not self.hide_date: 
            dtml = dtml + '  <th align="right" valign="top" class="list-item">Last Modified</th>\n'
        dtml = dtml + ' </tr>\n'
        cnt = 1 
        for item in self._getList(): 
            cnt = not cnt 
            icon, alt = self.getIcon(item) 
            id = item.getId() 
            title = item.title 
            if callable(title): title = title() 
            if not hasattr(item, 'getSize'): 
                size = ""
            else:
                size = getattr(item, 'getSize')()
                if size>1073741824:
                    size = "%.2f Gb" % (size / 1073741824)
                elif size>1048576:
                    size = "%.2f Mb" % (size / 1048576)
                elif size>1024:
                    size = "%.2f Kb" % (size / 1024)
                elif size==1:
                    size = "1 Byte"
                else:
                    size = "%d Bytes" % size
            date = item.bobobase_modification_time 
            if callable(date): date = date() 
            date = date.Date() 
            dtml = dtml + ' <tr class="row-%s">\n' % ((cnt and 'normal') or 'hilite') 
            dtml = dtml + '  <td align="left" valign="top" class="list-item" width="1%%"><a href="%s"><img src="%s" alt="%s" border="0" /></a></td>\n' % (item.absolute_url(), icon, alt, ) 
            if not self.hide_id: 
                dtml = dtml + '  <td align="left" valign="top" class="list-item"><a href="%s">%s</a></td>\n' % (item.absolute_url(), id or "&nbsp;", ) 
            if not self.hide_title: 
                dtml = dtml + '  <td align="left" valign="top" class="list-item"><a href="%s">%s</a></td>\n' % (item.absolute_url(), title or "&nbsp;", ) 
            if not self.hide_size: 
                dtml = dtml + '  <td align="right" valign="top" class="list-item">%s</td>\n' % (size or "&nbsp;", ) 
            if not self.hide_date: 
                dtml = dtml + '  <td align="right" valign="top" class="list-item">%s</td>\n' % (date or "&nbsp;", ) 
            dtml = dtml + ' </tr>\n'
        dtml = dtml + '</table>\n'
        return dtml

    def __call__(self):
        """ Display a rendered folder listing (With header/footer if selected) """
        if self.headers:
            dtml = "<dtml-var standard_html_header>\n"
        else:
            dtml = ""
        dtml = dtml + self._internals()
        if self.headers:
            dtml = dtml + "<dtml-var standard_html_footer>\n"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ContentIndex - __call__ - %s' % (self.getId(),)) 
        return obj(self.aq_inner, self.REQUEST) 

    def preview(self):
        """ Render a preview in the ZMI """
        dtml = "<dtml-var manage_page_header>\n<dtml-var manage_tabs>\n<br />\n"
	dtml = dtml + '<p class="form-help">See the online help for details of the CSS styles used by Content Index</p>\n'
        dtml = dtml + self._internals()
        dtml = dtml + "<dtml-var manage_page_footer>\n"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ContentIndex - preview - %s' % (self.getId(),))
        return obj(self.aq_inner, self.REQUEST)

    def getIcon(self, object):
        """ Find the cgi-bin file """
        icon = getattr(object, 'icon')
        if callable(icon): icon = icon()
        alt = getattr(object, 'meta_type')
        if callable(alt): alt = alt()
        if hasattr(object, 'content_type'):
            ctype = getattr(object, 'content_type')
            if callable(ctype): ctype = ctype()
            if hasattr(self, ContentIndexIcons.id):
                Icons = getattr(self, ContentIndexIcons.id)
                iconObj = Icons.getIconMapping(ctype)
                if iconObj:
                    icon = "%s/%s" % (Icons.getId(), iconObj.getId(), )
                    alt = ctype
        return icon, alt

def manage_addContentIndexIcons(self, REQUEST=None): 
    """ Add a ContentIndexIcons object """ 
    obj = ContentIndexIcons() 
    self._setObject(obj.getId(), obj) 
    if REQUEST is not None: 
        try: u=self.DestinationURL() 
        except: u=REQUEST['URL1'] 
        REQUEST.RESPONSE.redirect(u+'/manage_workspace') 
    return '' 
 
class ContentIndexIcons(
        OFS.Folder.Folder,
    ):
    """ Icons for the ContentIndex to acquire """
    id          =   "ContentIndexIcons"
    title       =   "Content Index Icon Folder"
    meta_type   =   "Content Index Icon Folder"
    icon        =   "misc_/ContentIndex/ContentIndex.gif"
    manage_icons=   Globals.HTMLFile("www/contentIcons", globals())
    manage_main =   manage_icons
    manage_options    = (
#        OFS.Folder.Folder.manage_options[:1] +
        (
            {'label': 'Icons', 'action': 'manage_icons',
             'help': ('ContentIndex', 'manage_icons.stx'), }, 
        ) +
        AccessControl.Role.RoleManager.manage_options +
        OFS.SimpleItem.Item.manage_options
    )

    def __init__(self):
        """ Initialize this ContentIndexIcons object """

    def getIconMapping(self, content_type):
        """ Find a icon based on content type (Title) """
        for item in self.objectValues():
            t = item.title
            if callable(t):
                t = t()
            if t == content_type:
                return item
        return None

